import { Component, Input, OnChanges, PipeTransform, TemplateRef } from '@angular/core';

export interface IFilterColumn {
  filterFunc?: (cells: any[]) => any;
  filterTemplate?: TemplateRef<any>;
  prop: string;
  pipe?: PipeTransform;
}

function defaultSumFunc(cells: any[]): any {
  return cells
    .filter(cell => !!cell)
    .reduce((res, cell) => res + cell);
}

@Component({
  selector: 'datatable-filter-row',
  template: `
  <datatable-body-row
    *ngIf="filterRow && _internalColumns"
    tabindex="-1"
    [innerWidth]="innerWidth"
    [offsetX]="offsetX"
    [columns]="_internalColumns"
    [rowHeight]="rowHeight"
    [row]="filterRow"
    [rowIndex]="-1">
  </datatable-body-row>
  `,
  host: {
    class: 'datatable-filter-row'
  }
})

export class DataTablefilterRowComponent implements OnChanges {
  @Input() rows: any[];
  @Input() columns: IFilterColumn[];

  @Input() rowHeight: number;
  @Input() offsetX: number;
  @Input() innerWidth: number;

  _internalColumns: IFilterColumn[];
  filterRow = {};

  ngOnChanges() {
    if (!this.columns || !this.rows) { return; }
    this.updateInternalColumns();
    this.updateValues();
  }

  private updateInternalColumns() {

    this._internalColumns = this.columns.map(col => ({
      ...col,
      cellTemplate: col.filterTemplate
    }));
  }

  private updateValues() {
    this.filterRow = {};
    this.columns
      .filter(col => !col.filterTemplate)
      .forEach(col => {
      const cellsFromSingleColumn = this.rows.map(row => row[col.prop]);
      const sumFunc = col.filterFunc || defaultSumFunc;
      this.filterRow[col.prop] = col.pipe ?
        col.pipe.transform(sumFunc(cellsFromSingleColumn)):
        sumFunc(cellsFromSingleColumn);
    });
  }
}
