import {Component, Output, EventEmitter, ChangeDetectionStrategy, Input, TemplateRef, ViewChild} from '@angular/core';
import {Http} from '@angular/http';
import {CookieService} from 'ngx-cookie-service';
import {Headers} from '@angular/http';

@Component({
    selector: 'datatable-footer',
    template: `
        <div
                class="datatable-footer-inner"
                [ngClass]="{'selected-count': selectedMessage}"
                [style.height.px]="footerHeight">
            <ng-template
                    *ngIf="footerTemplate"
                    [ngTemplateOutlet]="footerTemplate.template"
                    [ngTemplateOutletContext]="{ 
          rowCount: rowCount, 
          pageSize: pageSize, 
          selectedCount: selectedCount,
          curPage: curPage,
          offset: offset
        }">
            </ng-template>
            <div class="page-count" *ngIf="!footerTemplate">
        <span *ngIf="selectedMessage">
          {{selectedCount.toLocaleString()}} {{selectedMessage}} /
        </span>
                {{rowCount.toLocaleString()}} {{totalMessage}}
            </div>
            <datatable-pager *ngIf="!footerTemplate"
                             [pagerLeftArrowIcon]="pagerLeftArrowIcon"
                             [pagerRightArrowIcon]="pagerRightArrowIcon"
                             [pagerPreviousIcon]="pagerPreviousIcon"
                             [pagerNextIcon]="pagerNextIcon"
                             [page]="curPage"
                             [size]="pageSize"
                             [count]="rowCount"
                             [hidden]="!isVisible"
                             (change)="page.emit($event)">
            </datatable-pager>
            <ul style="display:flex ;   float: left;">
                <li style="display:   inline-block ;padding: 10px;;">
                    <select #selected (click)="onClick(selected.value)">
                        <option value="5">5</option>
                        <option value="10">10</option>
                        <option value="15">15</option>
                        <option value="20">20</option>
                    </select>
                </li>
                <li style="display:  inline-block;padding: 10px;">
                    <input type="image" src="http://uupload.ir/files/b1dp_pdf.png" height="18px" (click)="exportPdf()" 
                           style="height:18px ; width:18px"/>
                </li>
                <li style="display: inline-block;     padding: 10px;">
                    <input type="image" src="http://uupload.ir/files/v8vt_excel.png" (click)="exportExcel()" style="height:18px ; width:18px" />
                </li>
            </ul>
        </div>
    `,
    host: {
        class: 'datatable-footer'
    },
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class DataTableFooterComponent {
    columnsArray: any[];
    @Input() rows: any;
    @Input() appUrl: any;
    @Input() resource: any;
    @Input() sortDir: any;
    @Input() sortProp: any;
    @Input() includes: any;
    @Input() filters: any;
    @Input() columns: any;
    @Input() footerHeight: number;
    @Input() rowCount: number;
    @Input() pageSize: number;
    @Input() offset: number;
    @Input() pagerLeftArrowIcon: string;
    @Input() pagerRightArrowIcon: string;
    @Input() pagerPreviousIcon: string;
    @Input() pagerNextIcon: string;
    @Input() totalMessage: string;
    @Input() footerTemplate: TemplateRef<any>;
    @Input() selectedCount: number = 0;
    @Input() selectedMessage: string | boolean;
    @Output() page: EventEmitter<any> = new EventEmitter();
    // for test out put  for emit value of selected
    @Output() selectedValue = new EventEmitter();

    get isVisible(): boolean {
        return (this.rowCount / this.pageSize) > 1;
    }

    get curPage(): number {
        return this.offset + 1;
    }

    onClick(value: any) {
        this.selectedValue.emit(value);
    }

    @ViewChild('selected') selected: any;

    constructor(private http: Http,
                private cookieService: CookieService,) {
        this.columnsArray = [];
    }

    repairRows(arr: any[]): string {
        const rv: any[] = [];
        for (let i = 0; i < arr.length; ++i)
            if (arr[i] !== undefined) {
                rv.push(Object.assign({}, arr[i]));
                return JSON.stringify(rv);
            }
    }

    repaircolumns(arr: any[]): string {
        for (const key in arr) {
            const obj = {
                columns: this.columns[key]['prop'],
                name: this.columns[key]['name'],
            };
            this.columnsArray.push(obj);
        }
        return JSON.stringify(this.columnsArray);
    }

    exportExcel(): any {
        const token = this.cookieService.get('token');
        const headers = new Headers();
        headers.append('Content-Type', 'application/x-www-form-urlencoded\n');
        headers.append('Authorization', 'Bearer ' + token);
        // return this.http.post(this.appUrl + 'excel', 'type=' + 'excel' +
        //   '&filters=' + JSON.stringify(this.filters) + '&columns=' + this.repaircolumns(this.columns) + '&sortDir=' + this.sortDir +
        //   '&sortProp=' + this.sortProp + '&includes=' + this.includes + '&resource=' + this.resource,
        //   {headers: headers}
        // ).subscribe(x => {
        //     const blob = new Blob([x], { type: 'application/vnd.ms-excel' });
        //     const url= window.URL.createObjectURL(blob);
        //     window.open(url);
        //   }
        // );
        //****
        return this.http.post(this.appUrl + 'excel',
          'columns=' + this.repaircolumns(this.columns) ,{headers: headers}
        ).subscribe(x => {
            const blob = new Blob([x], { type: 'application/vnd.ms-excel' });
            const url= window.URL.createObjectURL(blob);
            window.open(url);
          }
        );
    }

    exportPdf(): any {
        const token = this.cookieService.get('token');
        const headers = new Headers();
        headers.append('Content-Type', 'application/force-download');
        headers.append('Authorization', 'Bearer ' + token);
        return this.http.post(this.appUrl + 'pdf', 'type=' + 'pdf' +
            '&filters=' + JSON.stringify(this.filters) + '&columns=' + this.repaircolumns(this.columns) + '&sortDir=' + this.sortDir +
            '&sortProp=' + this.sortProp + '&includes=' + this.includes + '&resource=' + this.resource
            , {headers: headers}
        ).subscribe(x => {
            // todo in test beshe
            const blob = new Blob([x], { type: 'application/vnd.ms-pdf' });
            const url= window.URL.createObjectURL(blob);
            window.open(url);
        });
    }
}
